@@ -3,9 +3,14 @@
 Prawn::Text::Formatted::Fragment.prepend (Module.new do
   attr_reader :document
 
-  # Prevent fragment from being written by discarding the text.
-  def conceal
-    @text = ''
+  # Prevent fragment from being written by discarding the text, optionally forcing the width to 0
+  def conceal force_width_to_zero = false
+    if force_width_to_zero
+      @width = 0
+      @text = ''
+    else
+      @text = ' ' * space_count # preserve space_count so width is still computed correctly
+    end
   end
 
   # Don't strip soft hyphens when repacking unretrieved fragments
@@ -14,6 +19,11 @@ Prawn::Text::Formatted::Fragment.prepend (Module.new do
     super
   end
 
+  # Use .tr instead of .gsub to remove zero-width spaces
+  def strip_zero_width_spaces string
+    string.encoding == Encoding::UTF_8 ? (string.tr Prawn::Text::ZWSP, '') : string
+  end
+
   # Modify the built-in ascender write method to allow an override value to be
   # specified using the format_state hash.
   def ascender= val
@@ -28,7 +38,7 @@ Prawn::Text::Formatted::Fragment.prepend (Module.new do
 
   def width
     if (val = format_state[:width])
-      val = (val.end_with? 'em') ? val.to_f * @document.font_size : (@document.str_to_pt val) if ::String === val
+      val = (val.end_with? 'em') ? val.to_f * @document.font_size : (@document.str_to_pt val) if String === val
     else
       val = super
     end
